# #  Copyright (C) 2025
# #  ABM, Moscow
# #
# #  UNPUBLISHED PROPRIETARY MATERIAL.
# #  ALL RIGHTS RESERVED.
# #
# #  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
# from dataclasses import dataclass, field
#
# from init_helpers import Jsonable
#
# from .body_part_description import BodyPartDescription
# from .spec_resource import SpecResource
#
#
# @dataclass(frozen=True, slots=True)
# class Content(SpecResource):
#     schema: type
#     encoding: dict[str, BodyPartDescription] = field(default_factory=dict)
#
#     def get_spec_dependencies(self) -> set['SpecResource']:
#         raise NotImplementedError
#
#     def get_spec_dict(self) -> dict[str, Jsonable]:
#         raise NotImplementedError
